import { useState } from 'react';

export type EditBundle<T = number> = {
  open: boolean;
  id?: T;
  create: () => void;
  update: (id: T) => void;
  close: () => void;
};

export function useEdit<T = number>(): EditBundle<T> {
  const [edit, setEdit] = useState<{ open: boolean; id?: T }>({ open: false, id: undefined });

  return {
    open: edit.open,
    id: edit.id,
    create: () => setEdit({ open: true, id: undefined }),
    update: (id: T) => setEdit({ open: true, id: id }),
    close: () => setEdit(() => ({ open: false, id: undefined })),
  };
}
