import React, { useRef } from 'react';
import { ActionType, PageContainer, ProTable } from '@ant-design/pro-components';
import { Card, Image, Popconfirm } from 'antd';
import { invoke } from '@/core/network';

type User = {
  id: number;
  mobile: string;
  nickname: string;
  avatarThumbnail: string;
  avatarOriginal: string;
};

type UserList = {
  records: User[];
  total: number;
  size: number;
  current: number;
};

class Service {
  public static async page(current: number, size: number) {
    return invoke<UserList>('/apis/admin/user/page', {
      method: 'POST',
      data: {
        current,
        size,
      },
    });
  }

  public static async remove(id: number) {
    return invoke<void>(`/apis/admin/user/remove?id=${id}`, {
      method: 'DELETE',
    });
  }

  public static async get(id: number) {
    return invoke<User>(`/apis/admin/user/details?id=${id}`, {
      method: 'GET',
    });
  }
}

const Users: React.FC<{}> = (props) => {
  const ref = useRef<ActionType>();

  return (
    <PageContainer>
      <Card>
        <ProTable<User>
          actionRef={ref}
          rowKey="id"
          search={false}
          toolbar={{
            actions: [],
            settings: [],
          }}
          request={async (params) => {
            const { current, pageSize } = params;
            const result = await Service.page(current || 1, pageSize || 20);
            return {
              data: result.data.records,
              total: result.data.total,
              success: result.success,
            };
          }}
          columns={[
            {
              title: 'ID',
              dataIndex: 'id',
            },
            {
              title: '头像',
              dataIndex: 'avatarThumbnail',
              render: (text, record) => {
                return <Image src={record.avatarThumbnail} width={50}></Image>;
              },
            },
            {
              title: '手机号码',
              dataIndex: 'mobile',
            },
            {
              title: '昵称',
              dataIndex: 'nickname',
            },
            {
              title: '操作',
              valueType: 'option',
              render: (_, record) => [
                <a
                  key={'edit'}
                  onClick={async () => {
                    const user = await Service.get(record.id);
                    console.log(user);
                  }}
                >
                  编辑
                </a>,
                <Popconfirm
                  key={'delete'}
                  title={'确认删除吗？'}
                  onConfirm={async () => {
                    await Service.remove(record.id);
                    ref.current?.reload();
                  }}
                >
                  <a key={'delete'}>删除</a>
                </Popconfirm>,
              ],
            },
          ]}
        ></ProTable>
      </Card>
    </PageContainer>
  );
};

export default Users;
